@@ -30,6 +30,10 @@
 #define _XOPEN_SOURCE 600 /* setenv */
 #endif
 
+#ifndef ESO_COVERAGE_BUILD
+#define ESO_COVERAGE_BUILD 0
+#endif
+
 #include "visir_recipe.h"
 
 #include "visir_utils.h"
@@ -99,7 +103,7 @@ static int visir_spc_reduce_exec(cpl_plugin * plugin)
         setenv("OMP_NUM_THREADS", "0", 1);
         return visir_tmpdir_exec(RECIPE_STRING, plugin, visir_spc_reduce);
     }
-    else if (getenv("VISIR_NO_FORK") != NULL) {
+    else if (ESO_COVERAGE_BUILD || getenv("VISIR_NO_FORK") != NULL) {
         /* debugging and coverage is better without fork */
         return cpl_recipedefine_exec(plugin, visir_spc_reduce)
             ? (int)cpl_error_set_where(cpl_func) : 0;
@@ -395,9 +399,6 @@ static int visir_spc_reduce(cpl_frameset            * framelist,
     if (cpl_frameset_find(repackset, VISIR_CALIB_STATIC_MASK)) {
         nrep--;
     }
-    if (cpl_frameset_find(repackset, VISIR_CALIB_STATIC_MASK_SPC)) {
-        nrep--;
-    }
     if (cpl_frameset_count_tags(repackset, VISIR_SPC_OBS_ECH_RAW) == nrep) {
         input_tag = VISIR_SPC_OBS_ECH_PP;
     }
