@@ -427,7 +427,8 @@ static int visir_old_spc_obs_ech(cpl_frameset            * framelist,
             irplib_framelist_get_propertylist_const(rawframes, 0);
         skip_if(0);
         if (visir_spc_phot_sensit(rawframes, &spc_config, plist, star_cat,
-                                  &weight2d, qclist, spc_table, dit_key)) {
+                                  &weight2d, qclist, spc_table, resol,
+                                  dit_key)) {
             irplib_error_recover(prestate, "Could not perform photometric "
                                  "calibration");
         } else {
