@@ -28,18 +28,54 @@
 #include <visir_inputs.h>
 
 #include "visir_inputs.c"
+/* for unlink */
+#include <unistd.h>
 
 /*-----------------------------------------------------------------------------
                                 Defines
  -----------------------------------------------------------------------------*/
 
-/* TEMPORARY SUPPORT OF CPL 5.x */
-#if defined CPL_VERSION_CODE && CPL_VERSION_CODE < CPL_VERSION(6, 0, 0)
-#define VISIR_ERROR_ACCESS_OUT_OF_RANGE CPL_ERROR_ILLEGAL_INPUT
-#else
-#define VISIR_ERROR_ACCESS_OUT_OF_RANGE CPL_ERROR_ACCESS_OUT_OF_RANGE
+#ifndef BOX_SIZE
+#define BOX_SIZE 11
 #endif
-/* END TEMPORARY SUPPORT OF CPL 5.x */
+
+#ifndef LINE_SIZE
+#define LINE_SIZE BOX_SIZE
+#endif
+
+#define TEST_LIST(list, value) \
+do { \
+    for (int i = 0; i < cpl_imagelist_get_size((list)); i++) { \
+        int b; \
+        const cpl_image * image = cpl_imagelist_get_const((list), i); \
+        cpl_test_eq((value), cpl_image_get(image, 1, 1, &b)); \
+    } \
+} \
+while(0)
+
+#define A_VAL 4.
+#define B_VAL 3.
+
+/*-----------------------------------------------------------------------------
+                         Declarations of private functions
+ -----------------------------------------------------------------------------*/
+
+static void visir_img_check_box_test(void);
+static void visir_img_check_align_test(void);
+static void visir_img_check_line_test(void);
+static void visir_check_get_type(void);
+static void visir_img_check_burst_find_delta_chop(void);
+static void visir_img_check_load_burst(void);
+static void visir_img_check_get_to_off_plane(void);
+static void visir_img_check_load_cube2(void);
+
+static cpl_propertylist * create_property_list(const char *,
+                                               const char *,
+                                               const double,
+                                               const double,
+                                               const double);
+static cpl_imagelist * create_cube2_imagelist(int);
+static cpl_imagelist * create_input_imagelist(const int *, int);
 
 
 /*----------------------------------------------------------------------------*/
@@ -54,25 +90,6 @@
    @brief   Unit tests of inputs module
 **/
 /*----------------------------------------------------------------------------*/
-
-static void visir_img_check_box_test(void);
-static void visir_img_check_align_test(void);
-static void visir_img_check_line_test(void);
-static void visir_img_check_burst_find_delta_chop(void);
-static void visir_img_check_load_burst(void);
-static void visir_img_check_get_to_off_plane(void);
-static void visir_img_check_load_cube2(void);
-
-void cpl_imagelist_empty(cpl_imagelist * self);
-
-#ifndef BOX_SIZE
-#define BOX_SIZE 11
-#endif
-
-#ifndef LINE_SIZE
-#define LINE_SIZE BOX_SIZE
-#endif
-
 int main(void)
 {
 
@@ -81,6 +98,7 @@ int main(void)
     visir_img_check_line_test();
     visir_img_check_align_test();
     visir_img_check_box_test();
+    visir_check_get_type();
     visir_img_check_burst_find_delta_chop();
     visir_img_check_load_burst();
     visir_img_check_get_to_off_plane();
@@ -90,6 +108,8 @@ int main(void)
 
 }
 
+/* FIXME: Write a short doxygen of each static function */
+
 static void visir_img_check_box_test(void)
 {
 
@@ -139,47 +159,47 @@ static void visir_img_check_box_test(void)
     cpl_apertures_dump(apneg, stream);
 
     /* Various error checks */
-    ecc = visir_img_check_box(NULL, 1, 2, apneg, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(NULL, 1, 2, apneg, 1, 2, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_NULL_INPUT);
 
-    ecc = visir_img_check_box(appos, 1, 2, NULL, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 1, 2, NULL, 1, 2, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_NULL_INPUT);
 
-    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, BOX_SIZE, 0.,
                               &swapp, NULL);
     cpl_test_error(CPL_ERROR_UNSPECIFIED);
 
-    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, BOX_SIZE, 0.,
                               NULL, &swapn);
     cpl_test_error(CPL_ERROR_UNSPECIFIED);
 
-    ecc = visir_img_check_box(appos, 1, 2, appos, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 1, 2, appos, 1, 2, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_UNSPECIFIED);
 
-    ecc = visir_img_check_box(appos, 0, 2, apneg, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 0, 2, apneg, 1, 2, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
-    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 0, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 0, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
-    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 3, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 3, BOX_SIZE, 0.,
                               &swapp, &swapn);
-    cpl_test_error(VISIR_ERROR_ACCESS_OUT_OF_RANGE);
+    cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
 
-    ecc = visir_img_check_box(appos, 3, 2, apneg, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 3, 2, apneg, 1, 2, BOX_SIZE, 0.,
                               &swapp, &swapn);
-    cpl_test_error(VISIR_ERROR_ACCESS_OUT_OF_RANGE);
+    cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
 
-    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, 0.0, &swapp, &swapn);
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, 0.0, 0., &swapp, &swapn);
     cpl_test_error(CPL_ERROR_UNSPECIFIED);
 
     /* Verify output of correct call - with zero eccentricity */
-    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_NONE);
 
@@ -188,7 +208,7 @@ static void visir_img_check_box_test(void)
     cpl_test_zero(swapn);
 
     /* - and points are swapped when needed */
-    ecc = visir_img_check_box(appos, 2, 1, apneg, 2, 1, BOX_SIZE,
+    ecc = visir_img_check_box(appos, 2, 1, apneg, 2, 1, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_NONE);
 
@@ -197,7 +217,7 @@ static void visir_img_check_box_test(void)
     cpl_test(swapn);
 
     /* Verify output of correct call - with maximum (?) eccentricity */
-    ecc = visir_img_check_box(apneg, 1, 2, appos, 1, 2, BOX_SIZE,
+    ecc = visir_img_check_box(apneg, 1, 2, appos, 1, 2, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_NONE);
 
@@ -206,7 +226,7 @@ static void visir_img_check_box_test(void)
     cpl_test(swapn);
 
     /* - and points are swapped when needed */
-    ecc = visir_img_check_box(apneg, 2, 1, appos, 2, 1, BOX_SIZE,
+    ecc = visir_img_check_box(apneg, 2, 1, appos, 2, 1, BOX_SIZE, 0.,
                               &swapp, &swapn);
     cpl_test_error(CPL_ERROR_NONE);
 
@@ -218,6 +238,96 @@ static void visir_img_check_box_test(void)
     cpl_apertures_delete(appos);
     cpl_apertures_delete(apneg);
 
+    /* verify with angle 45° */
+    image = cpl_image_new(600, 600, CPL_TYPE_INT);
+    double throw = 220;
+
+    /* Create four 3x3-pixel objects */
+    for (j = -1; j <= 1; j++) {
+      for (i = -1; i <= 1; i++) {
+          cpl_image_set(image, 350 + i,  80 + j,  1.0);
+          cpl_image_set(image, 350 + i, 400 + j,  1.0);
+          cpl_image_set(image, 190 + i, 240 + j, -1.0);
+          cpl_image_set(image, 510 + i, 240 + j, -1.0);
+      }
+    }
+
+    appos = cpl_apertures_extract_sigma(image, sigma);
+    cpl_image_multiply_scalar(image, -1.0);
+    apneg = cpl_apertures_extract_sigma(image, sigma);
+
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, throw,
+                              45. * CPL_MATH_RAD_DEG,
+                              &swapp, &swapn);
+    cpl_test_error(CPL_ERROR_NONE);
+
+    cpl_test_rel(ecc, 0.057, 0.1);
+    cpl_test(swapp);
+    cpl_test_zero(swapn);
+
+    cpl_image_delete(image);
+    cpl_apertures_delete(appos);
+    cpl_apertures_delete(apneg);
+
+    /* 30° */
+    image = cpl_image_new(600, 600, CPL_TYPE_INT);
+
+    for (j = -1; j <= 1; j++) {
+      for (i = -1; i <= 1; i++) {
+          cpl_image_set(image, 453 + i,  172 + j,  1.0);
+          cpl_image_set(image, 373 + i, 470 + j,  1.0);
+          cpl_image_set(image, 260 + i, 280 + j, -1.0);
+          cpl_image_set(image, 570 + i, 360 + j, -1.0);
+      }
+    }
+
+    appos = cpl_apertures_extract_sigma(image, sigma);
+    cpl_image_multiply_scalar(image, -1.0);
+    apneg = cpl_apertures_extract_sigma(image, sigma);
+
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, throw,
+                              30. * CPL_MATH_RAD_DEG,
+                              &swapp, &swapn);
+    cpl_test_error(CPL_ERROR_NONE);
+
+    cpl_test_rel(ecc, 0.049, 0.1);
+    cpl_test(swapp);
+    cpl_test_zero(swapn);
+
+    cpl_image_delete(image);
+    cpl_apertures_delete(appos);
+    cpl_apertures_delete(apneg);
+
+    /* 85° */
+    image = cpl_image_new(600, 600, CPL_TYPE_INT);
+
+    for (j = -1; j <= 1; j++) {
+      for (i = -1; i <= 1; i++) {
+          cpl_image_set(image, 363 + i,  396 + j,  1.0);
+          cpl_image_set(image, 156 + i, 159 + j,  1.0);
+          cpl_image_set(image, 138 + i, 382 + j, -1.0);
+          cpl_image_set(image, 381 + i, 175 + j, -1.0);
+      }
+    }
+
+    appos = cpl_apertures_extract_sigma(image, sigma);
+    cpl_image_multiply_scalar(image, -1.0);
+    apneg = cpl_apertures_extract_sigma(image, sigma);
+
+    ecc = visir_img_check_box(appos, 1, 2, apneg, 1, 2, throw,
+                              85. * CPL_MATH_RAD_DEG,
+                              &swapp, &swapn);
+    cpl_test_error(CPL_ERROR_NONE);
+
+    cpl_test_rel(ecc, 0.049, 0.1);
+    cpl_test(swapp);
+    /* fails, not required to be correct for aqu pipeline */
+    //cpl_test_zero(swapn);
+
+    cpl_image_delete(image);
+    cpl_apertures_delete(appos);
+    cpl_apertures_delete(apneg);
+
     if (stream != stdout) cpl_test_zero( fclose(stream) );
 
 }
@@ -244,6 +354,7 @@ static void visir_img_check_align_test(void)
         double          ecc;
         cpl_boolean     swapn, swapn2;
         int i, j;
+        double dhor = 90 * CPL_MATH_RAD_DEG;
 
 
         /* Create three 3x3-pixel objects */
@@ -285,67 +396,67 @@ static void visir_img_check_align_test(void)
         cpl_apertures_dump(apneg, stream);
 
         /* Various error checks */
-        ecc = visir_img_check_align(NULL, 1, apneg, 1, 2, LINE_SIZE, is_hor,
+        ecc = visir_img_check_align(NULL, 1, apneg, 1, 2, LINE_SIZE, 0.,
                                     &swapn);
         cpl_test_error(CPL_ERROR_NULL_INPUT);
 
-        ecc = visir_img_check_align(appos, 1, NULL, 1, 2, LINE_SIZE, is_hor,
+        ecc = visir_img_check_align(appos, 1, NULL, 1, 2, LINE_SIZE, 0.,
                                     &swapn);
         cpl_test_error(CPL_ERROR_NULL_INPUT);
 
-        ecc = visir_img_check_align(appos, 1, apneg, 1, 2, LINE_SIZE, is_hor,
+        ecc = visir_img_check_align(appos, 1, apneg, 1, 2, LINE_SIZE, 0.,
                                     NULL);
         cpl_test_error(CPL_ERROR_UNSPECIFIED);
 
-        ecc = visir_img_check_align(appos, 0, apneg, 1, 2, LINE_SIZE, is_hor,
+        ecc = visir_img_check_align(appos, 0, apneg, 1, 2, LINE_SIZE, 0.,
                                     &swapn);
         cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
-        ecc = visir_img_check_align(appos, 1, apneg, 1, 0, LINE_SIZE, is_hor,
+        ecc = visir_img_check_align(appos, 1, apneg, 1, 0, LINE_SIZE, 0.,
                                     &swapn);
         cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
-        ecc = visir_img_check_align(appos, 1, apneg, 0, 1, LINE_SIZE, is_hor,
+        ecc = visir_img_check_align(appos, 1, apneg, 0, 1, LINE_SIZE, 0.,
                                     &swapn);
         cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
-        ecc = visir_img_check_align(appos, 1, apneg, 1, 3, LINE_SIZE, is_hor,
+        ecc = visir_img_check_align(appos, 1, apneg, 1, 3, LINE_SIZE, 0.,
                                     &swapn);
-        cpl_test_error(VISIR_ERROR_ACCESS_OUT_OF_RANGE);
+        cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
 
-        ecc = visir_img_check_align(appos, 3, apneg, 1, 2, LINE_SIZE, is_hor,
+        ecc = visir_img_check_align(appos, 3, apneg, 1, 2, LINE_SIZE, 0.,
                                     &swapn);
-        cpl_test_error(VISIR_ERROR_ACCESS_OUT_OF_RANGE);
+        cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
 
-        ecc = visir_img_check_align(appos, 1, apneg, 1, 2, 0.0, is_hor, &swapn);
+        ecc = visir_img_check_align(appos, 1, apneg, 1, 2, 0.0, 0., &swapn);
         cpl_test_error(CPL_ERROR_UNSPECIFIED);
 
         /* Verify output of correct call - with zero eccentricity */
-        ecc = visir_img_check_align(appos, 1, apneg, 1, 2, LINE_SIZE, is_hor,
-                                    &swapn);
+        ecc = visir_img_check_align(appos, 1, apneg, 1, 2, LINE_SIZE,
+                                    is_hor ? dhor : 0., &swapn);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_abs(ecc, 0.0, FLT_EPSILON);
         cpl_test_zero(swapn);
 
         /* - and points are swapped when needed */
-        ecc = visir_img_check_align(appos, 1, apneg, 2, 1, LINE_SIZE, is_hor,
-                                    &swapn);
+        ecc = visir_img_check_align(appos, 1, apneg, 2, 1, LINE_SIZE,
+                                    is_hor ? dhor : 0., &swapn);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_abs(ecc, 0.0, FLT_EPSILON);
         cpl_test(swapn);
 
         /* Verify output of correct call - with maximum (?) eccentricity */
-        ecc = visir_img_check_align(appos, 1, apneg, 1, 2, LINE_SIZE, !is_hor,
-                                    &swapn);
+        ecc = visir_img_check_align(appos, 1, apneg, 1, 2, LINE_SIZE,
+                                     !is_hor ? dhor : 0., &swapn);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_abs(ecc, 2.0, FLT_EPSILON);
 
         /* - also with points swapped */
-        ecc = visir_img_check_align(appos, 1, apneg, 2, 1, LINE_SIZE, !is_hor,
-                                    &swapn2);
+        ecc = visir_img_check_align(appos, 1, apneg, 2, 1, LINE_SIZE,
+                                     !is_hor ? dhor : 0. ,&swapn2);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_abs(ecc, 2.0, FLT_EPSILON);
@@ -382,6 +493,7 @@ static void visir_img_check_line_test(void)
         cpl_apertures * apneg;
         double          ecc;
         int i, j;
+        double dhor = 90 * CPL_MATH_RAD_DEG;
 
 
         /* Create three 3x3-pixel objects */
@@ -423,50 +535,54 @@ static void visir_img_check_line_test(void)
         cpl_apertures_dump(apneg, stream);
 
         /* Various error checks */
-        ecc = visir_img_check_line(NULL, 1, apneg, 1, LINE_SIZE, is_hor);
+        ecc = visir_img_check_line(NULL, 1, apneg, 1, LINE_SIZE, 0.);
         cpl_test_error(CPL_ERROR_NULL_INPUT);
 
-        ecc = visir_img_check_line(appos, 1, NULL, 1, LINE_SIZE, is_hor);
+        ecc = visir_img_check_line(appos, 1, NULL, 1, LINE_SIZE, 0.);
         cpl_test_error(CPL_ERROR_NULL_INPUT);
 
-        ecc = visir_img_check_line(appos, 0, apneg, 1, LINE_SIZE, is_hor);
+        ecc = visir_img_check_line(appos, 0, apneg, 1, LINE_SIZE, 0.);
         cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
-        ecc = visir_img_check_line(appos, 1, apneg, 0, LINE_SIZE, is_hor);
+        ecc = visir_img_check_line(appos, 1, apneg, 0, LINE_SIZE, 0.);
         cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
-        ecc = visir_img_check_line(appos, 1, apneg, 0, LINE_SIZE, is_hor);
+        ecc = visir_img_check_line(appos, 1, apneg, 0, LINE_SIZE, 0.);
         cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
-        ecc = visir_img_check_line(appos, 1, apneg, 3, LINE_SIZE, is_hor);
-        cpl_test_error(VISIR_ERROR_ACCESS_OUT_OF_RANGE);
+        ecc = visir_img_check_line(appos, 1, apneg, 3, LINE_SIZE, 0.);
+        cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
 
-        ecc = visir_img_check_line(appos, 3, apneg, 1, LINE_SIZE, is_hor);
-        cpl_test_error(VISIR_ERROR_ACCESS_OUT_OF_RANGE);
+        ecc = visir_img_check_line(appos, 3, apneg, 1, LINE_SIZE, 0.);
+        cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
 
-        ecc = visir_img_check_line(appos, 1, apneg, 1, 0.0, is_hor);
+        ecc = visir_img_check_line(appos, 1, apneg, 1, 0.0, 0.);
         cpl_test_error(CPL_ERROR_UNSPECIFIED);
 
         /* Verify output of correct call - with zero eccentricity */
-        ecc = visir_img_check_line(apneg, 1, appos, 1, LINE_SIZE, is_hor);
+        ecc = visir_img_check_line(apneg, 1, appos, 1, LINE_SIZE,
+                                   is_hor ? dhor : 0.);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_abs(ecc, 0.0, FLT_EPSILON);
 
         /* Verify output of correct call - with maximum (?) eccentricity */
-        ecc = visir_img_check_line(appos, 1, apneg, 1, LINE_SIZE, is_hor);
+        ecc = visir_img_check_line(appos, 1, apneg, 1, LINE_SIZE,
+                                   is_hor ? dhor : 0.);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_abs(ecc, 2.0, FLT_EPSILON);
 
         /* Verify output of correct call - with maximum (?) eccentricity */
-        ecc = visir_img_check_line(appos, 1, apneg, 1, LINE_SIZE, !is_hor);
+        ecc = visir_img_check_line(appos, 1, apneg, 1, LINE_SIZE,
+                                   !is_hor ? dhor : 0.);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_abs(ecc, CPL_MATH_SQRT2, FLT_EPSILON);
 
         /* - also with points swapped */
-        ecc = visir_img_check_line(apneg, 1, appos, 1, LINE_SIZE, !is_hor);
+        ecc = visir_img_check_line(apneg, 1, appos, 1, LINE_SIZE,
+                                   !is_hor ? dhor : 0.);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_abs(ecc, CPL_MATH_SQRT2, FLT_EPSILON);
@@ -482,12 +598,206 @@ static void visir_img_check_line_test(void)
 
 }
 
-cpl_propertylist *
-create_property_list(const char * obs_start,
-                     const char * chop_start,
-                     const double chop_freq,
-                     const double dit,
-                     const double nditskip)
+static void load_frame(const char * fn, cpl_frame ** frm,
+                       cpl_propertylist ** plist, const int iext)
+{
+    *frm = cpl_frame_new();
+    cpl_frame_set_filename(*frm, fn);
+    *plist = cpl_propertylist_load(fn, iext);
+}
+
+
+static void visir_check_get_type(void)
+{
+
+    FILE          * stream;
+    const char * fn = "get_type_check.fits";
+
+    stream = cpl_msg_get_level() > CPL_MSG_INFO
+        ? fopen("/dev/null", "a") : stdout;
+
+    cpl_propertylist * plist;
+    cpl_imagelist * l;
+    cpl_frame * frm;
+    cpl_propertylist * lplist;
+    cpl_size next;
+    visir_data_type type;
+
+    /* CUBE1 */
+    {
+        l = cpl_imagelist_new();
+        plist = cpl_propertylist_new();
+        cpl_propertylist_append_string(plist, VISIR_PFITS_STRING_FRAME_TYPE,
+                                       "CUBE1");
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_CHOP_NCYCLES, 2);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NDIT, 10);
+        for (int i = 0; i < 4; i++)
+            cpl_imagelist_set(l, cpl_image_new(4, 4, CPL_TYPE_INT), i);
+        cpl_imagelist_save(l, fn, CPL_TYPE_INT, plist, CPL_IO_CREATE);
+        load_frame(fn, &frm, &lplist, 0);
+
+        visir_get_data_type(frm, lplist, &type, &next);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_eq(VISIR_DATA_CUBE1, type);
+        cpl_test_eq(0, next);
+
+        cpl_frame_delete(frm);
+        cpl_propertylist_delete(lplist);
+        cpl_imagelist_delete(l);
+        cpl_propertylist_delete(plist);
+    }
+
+    /* CUBE2 */
+    {
+        l = cpl_imagelist_new();
+        plist = cpl_propertylist_new();
+        cpl_propertylist_append_string(plist, VISIR_PFITS_STRING_FRAME_TYPE,
+                                       "CUBE2");
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_CHOP_NCYCLES, 5);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NDIT, 10);
+        for (int i = 0; i < 5 * 2 + 1; i++)
+            cpl_imagelist_set(l, cpl_image_new(4, 4, CPL_TYPE_INT), i);
+        cpl_imagelist_save(l, fn, CPL_TYPE_INT, plist, CPL_IO_CREATE);
+        load_frame(fn, &frm, &lplist, 0);
+
+        visir_get_data_type(frm, lplist, &type, &next);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_eq(VISIR_DATA_CUBE2, type);
+        cpl_test_eq(0, next);
+
+        cpl_frame_delete(frm);
+        cpl_propertylist_delete(lplist);
+        cpl_imagelist_delete(l);
+        cpl_propertylist_delete(plist);
+    }
+
+    /* BURST */
+    {
+        l = cpl_imagelist_new();
+        plist = cpl_propertylist_new();
+        cpl_propertylist_append_string(plist, VISIR_PFITS_STRING_FRAME_TYPE,
+                                       "UNDEFINED");
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_CHOP_NCYCLES, 5);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NDIT, 10);
+        for (int i = 0; i < 5 * 2 * 10; i++)
+            cpl_imagelist_set(l, cpl_image_new(4, 4, CPL_TYPE_INT), i);
+        cpl_imagelist_save(l, fn, CPL_TYPE_INT, plist, CPL_IO_CREATE);
+        load_frame(fn, &frm, &lplist, 0);
+
+        visir_get_data_type(frm, lplist, &type, &next);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_eq(VISIR_DATA_BURST, type);
+        cpl_test_eq(0, next);
+
+        cpl_frame_delete(frm);
+        cpl_propertylist_delete(lplist);
+        cpl_imagelist_delete(l);
+        cpl_propertylist_delete(plist);
+    }
+
+    /* BURST incomplete*/
+    {
+        l = cpl_imagelist_new();
+        plist = cpl_propertylist_new();
+        cpl_propertylist_append_string(plist, VISIR_PFITS_STRING_FRAME_TYPE,
+                                       "UNDEFINED");
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_CHOP_NCYCLES, 5);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NDIT, 10);
+        for (int i = 0; i < 5 * 2 * 10 - 10; i++)
+            cpl_imagelist_set(l, cpl_image_new(4, 4, CPL_TYPE_INT), i);
+        cpl_imagelist_save(l, fn, CPL_TYPE_INT, plist, CPL_IO_CREATE);
+        load_frame(fn, &frm, &lplist, 0);
+
+        visir_get_data_type(frm, lplist, &type, &next);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_eq(VISIR_DATA_BURST, type);
+        cpl_test_eq(0, next);
+
+        cpl_frame_delete(frm);
+        cpl_propertylist_delete(lplist);
+        cpl_imagelist_delete(l);
+        cpl_propertylist_delete(plist);
+    }
+
+    /* AQU HCYCLE */
+    {
+        cpl_image * img = cpl_image_new(4, 4, CPL_TYPE_INT);
+        plist = cpl_propertylist_new();
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NAVRG, 1);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_CHOP_NCYCLES, 5);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NDIT, 10);
+        cpl_propertylist_save(plist, fn, CPL_IO_CREATE);
+        for (int i = 0; i < 5 * 2 + 1; i++)
+            cpl_image_save(img, fn, CPL_TYPE_INT, NULL, CPL_IO_EXTEND);
+        load_frame(fn, &frm, &lplist, 0);
+
+        visir_get_data_type(frm, lplist, &type, &next);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_eq(VISIR_DATA_AQU_HCYCLE, type);
+        cpl_test_eq(11, next);
+
+        cpl_frame_delete(frm);
+        cpl_propertylist_delete(lplist);
+        cpl_image_delete(img);
+        cpl_propertylist_delete(plist);
+    }
+
+    /* AQU BURST */
+    {
+        plist = cpl_propertylist_new();
+        l = cpl_imagelist_new();
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NAVRG, 1);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_CHOP_NCYCLES, 5);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NDIT, 10);
+        for (int i = 0; i < 5 * 2 * 10; i++)
+            cpl_imagelist_set(l, cpl_image_new(4, 4, CPL_TYPE_INT), i);
+        cpl_imagelist_save(l, fn, CPL_TYPE_INT, plist, CPL_IO_CREATE);
+        load_frame(fn, &frm, &lplist, 0);
+
+        visir_get_data_type(frm, lplist, &type, &next);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_eq(VISIR_DATA_AQU_BURST, type);
+        cpl_test_eq(0, next);
+
+        cpl_frame_delete(frm);
+        cpl_propertylist_delete(lplist);
+        cpl_imagelist_delete(l);
+        cpl_propertylist_delete(plist);
+    }
+    
+    /* AQU BURST EXT */
+    {
+        plist = cpl_propertylist_new();
+        l = cpl_imagelist_new();
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NAVRG, 1);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_CHOP_NCYCLES, 5);
+        cpl_propertylist_append_int(plist, VISIR_PFITS_INT_NDIT, 10);
+        cpl_propertylist_save(plist, fn, CPL_IO_CREATE);
+        for (int i = 0; i < 5 * 2 * 10; i++)
+            cpl_imagelist_set(l, cpl_image_new(4, 4, CPL_TYPE_INT), i);
+        cpl_imagelist_save(l, fn, CPL_TYPE_INT, NULL, CPL_IO_EXTEND);
+        load_frame(fn, &frm, &lplist, 0);
+
+        visir_get_data_type(frm, lplist, &type, &next);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_eq(VISIR_DATA_AQU_BURST_EXT, type);
+        cpl_test_eq(1, next);
+
+        cpl_frame_delete(frm);
+        cpl_propertylist_delete(lplist);
+        cpl_imagelist_delete(l);
+        cpl_propertylist_delete(plist);
+    }
+
+    unlink(fn);
+    if (stream != stdout) cpl_test_zero( fclose(stream) );
+}
+
+static cpl_propertylist * create_property_list(const char * obs_start,
+                                               const char * chop_start,
+                                               const double chop_freq,
+                                               const double dit,
+                                               const double nditskip)
 {
     cpl_propertylist * prop = cpl_propertylist_new();
     cpl_propertylist_append_string(prop, VISIR_PFITS_STRING_OBS_START,
@@ -505,7 +815,6 @@ static void visir_img_check_burst_find_delta_chop(void)
 {
 
     FILE          * stream;
-    cpl_error_code  error = CPL_ERROR_NONE;
 
     stream = cpl_msg_get_level() > CPL_MSG_INFO
         ? fopen("/dev/null", "a") : stdout;
@@ -553,18 +862,7 @@ static void visir_img_check_burst_find_delta_chop(void)
 
 }
 
-#define TEST_LIST(list, value) \
-do { \
-    for (int i = 0; i < cpl_imagelist_get_size((list)); i++) { \
-        int b; \
-        const cpl_image * image = cpl_imagelist_get_const((list), i); \
-        cpl_test_eq((value), cpl_image_get(image, 1, 1, &b)); \
-    } \
-} \
-while(0)
-
-cpl_imagelist *
-create_input_imagelist(const int * values, int naxis3)
+static cpl_imagelist * create_input_imagelist(const int * values, int naxis3)
 {
     cpl_imagelist * list = cpl_imagelist_new();
     for (int i = 0; i < naxis3; i++) {
@@ -579,7 +877,6 @@ create_input_imagelist(const int * values, int naxis3)
 static void visir_img_check_load_burst(void)
 {
     FILE          * stream;
-    cpl_error_code  error = CPL_ERROR_NONE;
 
     stream = cpl_msg_get_level() > CPL_MSG_INFO
         ? fopen("/dev/null", "a") : stdout;
@@ -720,7 +1017,6 @@ static void visir_img_check_load_burst(void)
     {
         cpl_imagelist * blist = cpl_imagelist_new();
         cpl_imagelist * alist = cpl_imagelist_new();
-        cpl_propertylist * prop;
         const int values[] = {0, 1, 1, 0, 0, 0xBAD, 0, 1, 1, 0, 0, 0xBAD, 0, 1};
         int naxis3 = sizeof(values)/sizeof(values[0]);
         int ichopchange = 5, ihalfcycle = 6;
@@ -769,6 +1065,68 @@ static void visir_img_check_load_burst(void)
         cpl_propertylist_delete(prop);
     }
 
+    {
+        cpl_imagelist * blist = cpl_imagelist_new();
+        cpl_imagelist * alist = cpl_imagelist_new();
+        cpl_propertylist * prop;
+        const int values[] = {0xBADBEEF, 0xB, 0xB, 0xB, 0xB, 0xBADBEEF,
+                              0xA, 0xA, 0xA, 0xA, 0xBADBEEF, 0xB};
+        int naxis3 = sizeof(values)/sizeof(values[0]);
+        int ichopchange = 0, ihalfcycle = 0;
+        cpl_imagelist * list = create_input_imagelist(values, naxis3);
+
+        // halfcycle 5, chopstart 0
+        prop = create_property_list("2011-08-13T23:10:00.0000",
+                                    "2011-08-13T23:10:00", 0.1, 1, 0);
+        visir_img_burst_find_delta_chop(prop, &ichopchange, &ihalfcycle);
+        cpl_test_eq(0, ichopchange);
+        cpl_test_eq(5, ihalfcycle);
+
+        visir_load_burst_(alist, blist, list, ichopchange, ihalfcycle, 0, 0);
+        cpl_test_eq(4, cpl_imagelist_get_size(alist));
+        cpl_test_eq(5, cpl_imagelist_get_size(blist));
+
+        TEST_LIST(alist, 0xA);
+        TEST_LIST(blist, 0xB);
+
+        cpl_imagelist_delete(list);
+        cpl_imagelist_delete(alist);
+        cpl_imagelist_delete(blist);
+        cpl_propertylist_delete(prop);
+    }
+
+    {
+        const char * fn = "load_burst.fits";
+        cpl_imagelist * blist = cpl_imagelist_new();
+        cpl_imagelist * alist = cpl_imagelist_new();
+        cpl_propertylist * prop = cpl_propertylist_new();
+        cpl_frame * frame = cpl_frame_new();
+        const int values[] = {0xA, 0xA, 0xA, 0xA, 0xA, 0xA, 0xA, 0xA, 0xA};
+        int naxis3 = sizeof(values)/sizeof(values[0]);
+        cpl_imagelist * list = create_input_imagelist(values, naxis3);
+
+        cpl_propertylist_append_bool(prop, VISIR_PFITS_BOOL_CHOP_STATUS, CPL_FALSE);
+        cpl_imagelist_save(list, fn, CPL_TYPE_UNSPECIFIED,
+                           prop, CPL_IO_CREATE);
+        cpl_propertylist_delete(prop);
+        prop = cpl_propertylist_load(fn, 0);
+        cpl_frame_set_filename(frame, fn);
+
+        // TODO reenable when chop st is fixed
+        //visir_load_burst(alist, blist, frame, prop, 0, naxis3, 0, naxis3, 0, 0);
+        //cpl_test_eq(naxis3, cpl_imagelist_get_size(alist));
+        //cpl_test_eq(0, cpl_imagelist_get_size(blist));
+
+        TEST_LIST(alist, 0xA);
+
+        cpl_imagelist_delete(list);
+        cpl_imagelist_delete(alist);
+        cpl_imagelist_delete(blist);
+        cpl_propertylist_delete(prop);
+        cpl_frame_delete(frame);
+        unlink(fn);
+    }
+
     if (stream != stdout) cpl_test_zero( fclose(stream) );
 
 }
@@ -778,7 +1136,6 @@ static void visir_img_check_get_to_off_plane(void)
 {
 
     FILE          * stream;
-    cpl_error_code  error = CPL_ERROR_NONE;
 
     stream = cpl_msg_get_level() > CPL_MSG_INFO
         ? fopen("/dev/null", "a") : stdout;
@@ -786,7 +1143,8 @@ static void visir_img_check_get_to_off_plane(void)
     cpl_test_nonnull( stream );
 
     {
-        const int icc = 27;
+        /* cc < hc tests */
+        int icc = 27;
         int istart = 0;
         int ihc = 100;
         cpl_test_eq(27, get_to_off_plane(icc, istart, ihc));
@@ -821,32 +1179,72 @@ static void visir_img_check_get_to_off_plane(void)
         istart = 50;
         ihc = 80;
         cpl_test_eq(139, get_to_off_plane(29, istart, ihc));
+
+        /* cc > hc tests */
+        icc = 157;
+        istart = 240;
+        ihc = 80;
+        cpl_test_eq(317 - istart, get_to_off_plane(icc, istart, ihc));
+
+        istart = 230;
+        cpl_test_eq(317 - istart, get_to_off_plane(icc, istart, ihc));
+
+        istart = 160;
+        cpl_test_eq(317 - istart, get_to_off_plane(icc, istart, ihc));
+
+        istart = 150;
+        cpl_test_eq(157 - istart, get_to_off_plane(icc, istart, ihc));
+
+        istart = 320;
+        cpl_test_eq(477 - istart, get_to_off_plane(icc, istart, ihc));
+
+        istart = 310;
+        cpl_test_eq(317 - istart, get_to_off_plane(icc, istart, ihc));
+
+        /* cc == hc tests */
+        icc = 80;
+        istart = 0;
+        ihc = 80;
+        cpl_test_eq(80, get_to_off_plane(icc, istart, ihc));
+
+        istart = 80;
+        cpl_test_eq(0, get_to_off_plane(icc, istart, ihc));
+
+        istart = 160;
+        cpl_test_eq(80, get_to_off_plane(icc, istart, ihc));
+
+        istart = 240;
+        cpl_test_eq(0, get_to_off_plane(icc, istart, ihc));
+
+        istart = 30;
+        cpl_test_eq(50, get_to_off_plane(icc, istart, ihc));
+
+        istart = 130;
+        cpl_test_eq(110, get_to_off_plane(icc, istart, ihc));
     }
 
     if (stream != stdout) cpl_test_zero( fclose(stream) );
 }
 
-#define A_VAL 4.
-#define B_VAL 3.
-
 /* creates cube2 formated list, length 2*n + 1 */
-cpl_imagelist * create_cube2_imagelist(int ncycles)
+static cpl_imagelist * create_cube2_imagelist(int ncycles)
 {
     cpl_imagelist * list = cpl_imagelist_new();
-    float prev_val = 0;
-    for (int i = 0; i < ncycles; i++) {
-        cpl_image * image = cpl_image_new(3, 3, CPL_TYPE_FLOAT);
-        cpl_image_set(image, 1, 1, A_VAL);
-        cpl_imagelist_set(list, image, cpl_imagelist_get_size(list));
+    cpl_image * image = cpl_image_new(3, 3, CPL_TYPE_FLOAT);
+    float prev_val = 0.0;
 
+    for (int i = 0; i < ncycles; i++) {
+        const float val = (prev_val * i + A_VAL - B_VAL) / (i + 1);
+        cpl_image_set(image, 1, 1, A_VAL - VISIR_HCYCLE_OFFSET);
+        cpl_imagelist_set(list, image, cpl_imagelist_get_size(list));
         image = cpl_image_new(3, 3, CPL_TYPE_FLOAT);
-        float val = (prev_val * i + A_VAL - B_VAL) / (i + 1);
+
         cpl_image_set(image, 1, 1, val);
         cpl_imagelist_set(list, image, cpl_imagelist_get_size(list));
+        image = cpl_image_new(3, 3, CPL_TYPE_FLOAT);
         prev_val = val;
     }
 
-    cpl_image * image = cpl_image_new(3, 3, CPL_TYPE_FLOAT);
     cpl_image_set(image, 1, 1, prev_val);
     cpl_imagelist_set(list, image, cpl_imagelist_get_size(list));
     return list;
@@ -856,7 +1254,6 @@ static void visir_img_check_load_cube2(void)
 {
 
     FILE          * stream;
-    cpl_error_code  error = CPL_ERROR_NONE;
 
     stream = cpl_msg_get_level() > CPL_MSG_INFO
         ? fopen("/dev/null", "a") : stdout;
@@ -891,9 +1288,10 @@ static void visir_img_check_load_cube2(void)
         cpl_imagelist * list = create_cube2_imagelist(ncycles);
         cpl_imagelist * alist = cpl_imagelist_new();
         cpl_imagelist * blist = cpl_imagelist_new();
-        cpl_test_eq(2 * ncycles + 1, cpl_imagelist_get_size(list));
         cpl_imagelist * chunk2 = cpl_imagelist_new();
 
+        cpl_test_eq(2 * ncycles + 1, cpl_imagelist_get_size(list));
+
         /* strip the sum */
         cpl_image_delete(cpl_imagelist_unset(list,
                                              cpl_imagelist_get_size(list) - 1));
