@@ -45,7 +45,7 @@
 #define PIXEL_TYPE double
 #define STDEV_TYPE CPL_TYPE_DOUBLE
 #define PIXEL_TYPE_CPL CPL_TYPE_DOUBLE
-static size_t prnok_nz = 0;
+static int prnok_nz = 0;
 static double * prnok = NULL;
 #include "../recipes/visir_util_clip_body.c"
 #undef PIXEL_TYPE
@@ -1067,7 +1067,7 @@ static void visir_next_regular_test(void)
 
     for (cpl_size i = 0; i < 2048; i++) {
         size_t r = visir_get_next_regular(i);
-        cpl_test(r >= i);
+        cpl_test(r >= (size_t)i);
         size_t d = 2;
         int large_primes = 0;
         while(r > 1) {
@@ -1104,13 +1104,13 @@ static void visir_test_parallel_median(void)
         for (size_t i = 0; i < n; i++) {
             cpl_image * img = cpl_image_new(nx, ny, CPL_TYPE_FLOAT);
             float * d = cpl_image_get_data_float(img);
-            for (size_t i = 0; i < nx * ny; i++) {
+            for (size_t k = 0; k < (size_t)(nx * ny); k++) {
                 d[i] = rand() % nx;
                 /* fully bad timeline */
                 cpl_image_reject(img, 1, 1);
                 /* random bad pixels */
                 if ((rand() % 25) == 0) {
-                    cpl_image_reject(img, i % nx + 1, i / nx + 1);
+                    cpl_image_reject(img, k % nx + 1, k / nx + 1);
                 }
             }
             cpl_imagelist_set(l, img, i);
